Package: DMRcaller
Type: Package
Title: Differentially Methylated Regions Caller
Version: 1.45.3
Date: 2019-01-18
Encoding: UTF-8
Authors@R: c(person(given="Nicolae Radu", family="Zabet", role =
        c("aut","cre"), email = "r.zabet@qmul.ac.uk"),
        person(given="Jonathan", family="Tsang", role = "aut", email =
        "jmft2@cam.ac.uk"), person(given="Alessandro", family="Greco",
        role = "aut", email = "apgrec@essex.ac.uk"), person(given="Ryan",
        family="Merritt", role = "aut", email = "rmerri@essex.ac.uk"), person(given="Young Jun",
        family="Kim", role = "aut", email = "qc25039@qmul.ac.uk"))
Author: Nicolae Radu Zabet <r.zabet@qmul.ac.uk>, Jonathan
        Michael Foonlan Tsang <jmft2@cam.ac.uk>, Alessandro
        Pio Greco <apgrec@essex.ac.uk>, Ryan Merritt <rmerri@essex.ac.uk> and
        Young Jun Kim <qc25039@qmul.ac.uk>
Maintainer: Nicolae Radu Zabet <r.zabet@qmul.ac.uk>
Description: Uses Bisulfite sequencing data in two conditions and
        identifies differentially methylated regions between the
        conditions in CG and non-CG context. The input is the CX report
        files produced by Bismark and the output is a list of DMRs
        stored as GRanges objects.
License: GPL-3
LazyLoad: yes
Imports: parallel, Rcpp, RcppRoll, betareg, grDevices, graphics,
        methods, stats, utils, Rsamtools, GenomicRanges,
        GenomicAlignments, Biostrings, BSgenome, BiocManager,
        S4Vectors, IRanges, InteractionSet, stringr, inflection,
        BiocParallel, Seqinfo, GenomeInfoDb, cigarillo
Depends: R (>= 3.5), GenomicRanges, IRanges, S4Vectors
Suggests: knitr, RUnit, BiocGenerics, rmarkdown, bookdown, BiocStyle,
        betareg, rtracklayer, BSgenome.Hsapiens.UCSC.hg38
biocViews: DifferentialMethylation, DNAMethylation, Software,
        Sequencing, Coverage
VignetteBuilder: knitr
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/DMRcaller
git_branch: devel
git_last_commit: cae718d
git_last_commit_date: 2026-07-27
Repository: Bioconductor 3.24
Date/Publication: 2026-07-27
Packaged: 2026-07-27 21:25:35 UTC; biocbuild
