# Generated by roxygen2: do not edit by hand

export(analyseReadsInsideRegionsForCondition)
export(analyseReadsInsideRegionsForConditionPMD)
export(computeCoMethylatedPositions)
export(computeCoMethylatedRegions)
export(computeDMRs)
export(computeDMRsReplicates)
export(computeMethylationDataCoverage)
export(computeMethylationDataSpatialCorrelation)
export(computeMethylationProfile)
export(computeOverlapProfile)
export(computePMDs)
export(computeVMDs)
export(filterDMRs)
export(filterPMDs)
export(filterVMDs)
export(filterVMRsONT)
export(getWholeChromosomes)
export(joinReplicates)
export(mergeDMRsIteratively)
export(mergePMDsIteratively)
export(plotLocalMethylationProfile)
export(plotMethylationDataCoverage)
export(plotMethylationDataSpatialCorrelation)
export(plotMethylationProfile)
export(plotMethylationProfileFromData)
export(plotOverlapProfile)
export(poolMethylationDatasets)
export(poolTwoMethylationDatasets)
export(readBismark)
export(readBismarkPool)
export(readONTbam)
export(saveBismark)
export(selectCytosine)
import(BiocParallel)
import(Biostrings)
import(GenomicAlignments)
import(GenomicRanges)
import(IRanges)
import(InteractionSet)
import(Rcpp)
import(Rsamtools)
import(S4Vectors)
import(betareg)
import(inflection)
import(stringr)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,matchPattern)
importFrom(Biostrings,reverseComplement)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,mcols)
importFrom(IRanges,IRanges)
importFrom(IRanges,IntegerList)
importFrom(RcppRoll,roll_sum)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(methods,is)
importFrom(parallel,mclapply)
importFrom(stats,cor.test)
importFrom(stats,na.pass)
importFrom(stats,pnorm)
importFrom(stats,setNames)
importFrom(stats,var.test)
importFrom(stats,wilcox.test)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,write.table)
