% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.smartMergeDMRsReplicates}
\alias{.smartMergeDMRsReplicates}
\title{This takes a list of DMRs and attempts to merge DMRs while keeping the new
DMRs statistically significant}
\usage{
.smartMergeDMRsReplicates(
  DMRs,
  minGap,
  respectSigns = TRUE,
  methylationData,
  minProportionDifference = 0.4,
  minReadsPerCytosine = 4,
  pseudocountM = 1,
  pseudocountN = 2,
  pValueThreshold = 0.01,
  condition = condition,
  m = m,
  n = n,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This takes a list of DMRs and attempts to merge DMRs while keeping the new
DMRs statistically significant
}
