% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.countCytosinesInside}
\alias{.countCytosinesInside}
\title{Count cytosines inside}
\usage{
.countCytosinesInside(methylationData, regions)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata 
columns; see \code{\link{methylationDataList}}}

\item{regions}{a \code{\link[GenomicRanges]{GRanges}} with the computed regions}
}
\value{
a \code{vector} with the number of cytosines in each DMR
}
\description{
This function counts the number of cytosines in each DMR
}
\author{
Jonathan Michael Foonlan Tsang
}
