% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateGenome}
\alias{.validateGenome}
\title{Validate BSgenome Object}
\usage{
.validateGenome(genome)
}
\arguments{
\item{genome}{A BSgenome object (e.g. BSgenome.Hsapiens.UCSC.hg38).}
}
\value{
Invisibly `TRUE` if valid; otherwise throws an error.
}
\description{
Ensures `genome` inherits from BSgenome and that its package is installed.
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
