% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.mergeDMRsIteratively}
\alias{.mergeDMRsIteratively}
\title{This takes a list of DMRs and attempts to merge DMRs while keeping the new 
DMRs statistically significant}
\usage{
.mergeDMRsIteratively(
  DMRs,
  minGap,
  respectSigns = TRUE,
  methylationData,
  minProportionDifference = 0.4,
  minReadsPerCytosine = 4,
  pValueThreshold = 0.01,
  test = "fisher",
  alternative = "two.sided"
)
}
\description{
This takes a list of DMRs and attempts to merge DMRs while keeping the new 
DMRs statistically significant
}
