% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{GEs}
\alias{GEs}
\title{The genetic elements data}
\format{
A \code{GRanges} object
}
\source{
The object was created by calling \code{import.gff3} function
from \code{rtracklayer} package for
\url{ftp://ftp.arabidopsis.org/Maps/gbrowse_data/TAIR10/TAIR10_GFF3_genes_transposons.gff}
}
\description{
A \code{GRanges} object containing the annotation of the Arabidopsis thaliana
}
