% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{syntheticDataReplicates}
\alias{syntheticDataReplicates}
\title{Simulated data for biological replicates}
\format{
A \code{\link[GenomicRanges]{GRanges}} object containing multiple metadata
columns with the reads from each object passed as parameter
}
\source{
The object was created by calling \code{\link{joinReplicates}}
 function.
}
\description{
A \code{GRanges} object containing simulated date for methylation in four
samples. The conditions assciated witch each sample are a, a, b and b.
}
