% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{.computeAdjuestedPValues}
\alias{.computeAdjuestedPValues}
\title{Compute adjusted p-values}
\usage{
.computeAdjuestedPValues(test, m1, n1, m2, n2, alternative = "two.sided")
}
\arguments{
\item{test}{a \code{character} indicating the statistical test used.
(\code{"fisher"} for Fisher's exact test or \code{"score"} for Score test).}

\item{m1}{the number of methylated reads in condition 1.}

\item{n1}{the total number of methylated reads in condition 1.}

\item{m2}{the number of methylated reads in condition 2.}

\item{n2}{the total number of methylated reads in condition 2.}

\item{alternative}{indicates the alternative hypothesis and must be one of
\code{"two.sided"}, \code{"greater"} or \code{"less"}. You can specify just
the initial letter. Only used in the 2 by 2 case. This is used only for
Fisher's test}
}
\value{
The adjusted p-values of the statistical test.
}
\description{
This function computes the adjusted p-values (using Benjamini & Hochberg
method)
}
\author{
Radu Zabet
}
