% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.validateConditionReplicates}
\alias{.validateConditionReplicates}
\title{Validate the condition of the experiment}
\usage{
.validateConditionReplicates(condition, methylationData)
}
\arguments{
\item{condition}{The vector containing the two conditions for the experiment.}
}
\description{
Checks whether the passed parameter is a valid condition for the data
}
\author{
Alessandro Pio Greco and Nicolae Radu Zabet
}
