% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeProportionsInDMRs}
\alias{.computeProportionsInDMRs}
\title{This function recalculate proportions between methylated and total reads in a new region.}
\usage{
.computeProportionsInDMRs(DMRs, methylationData, col_indexes)
}
\description{
This function recalculate proportions between methylated and total reads in a new region.
}
