% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{GEs_hg38}
\alias{GEs_hg38}
\title{The genetic elements data of GRCh38 Genome Reference}
\format{
A \code{GRanges} object
}
\source{
The object was created by loading the \code{gtf} file from
\url{https://hgdownload.soe.ucsc.edu/goldenPath/hg38/bigZips/genes/hg38.refGene.gtf.gz}.
and concatenated by range in 1.5 ~ 2 Mbp from Chromosome 1
}
\description{
A \code{GRanges} object containing the annotation of the Homo sapiens (hg38)
}
