% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.isColor}
\alias{.isColor}
\title{Is color}
\usage{
.isColor(x, minLength = NULL)
}
\arguments{
\item{x}{the vector to be validated}

\item{minLength}{the  minimum length of the vector. If NULL the minimum
length is 1}
}
\value{
a \code{logical} value indicating whether \code{x} is a vector
containing only colors
}
\description{
This function checks whether the argument is a vector containing colours
}
\author{
Radu Zabet
}
