% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeDMRsReplicatesBins}
\alias{.computeDMRsReplicatesBins}
\title{This function computes the differentially methylated regions between replicates
using the bins method.}
\usage{
.computeDMRsReplicatesBins(
  methylationData,
  condition = condition,
  regions = NULL,
  context = "CG",
  binSize = 100,
  pseudocountM = 1,
  pseudocountN = 2,
  pValueThreshold = 0.01,
  minCytosinesCount = 4,
  minProportionDifference = 0.4,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  BPPARAM = BPPARAM,
  cores = cores
)
}
\description{
This function computes the differentially methylated regions between replicates
using the bins method.
}
