% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{scanBamChr1Random5}
\alias{scanBamChr1Random5}
\title{The bam file from ONT nanopore .pod5 files}
\format{
A \code{.bam} object
}
\source{
The object was created by base-calling and alignment by \code{dorado, ver.0.9.6}
\url{https://github.com/nanoporetech/dorado?tab=readme-ov-file#dna-models}
to generate the \code{bam} files with \code{dna_r10.4.1_e8.2_400bps_hac@v5.2.0} as basecalling model
from GM18501 cell line \url{https://s3.amazonaws.com/1000g-ont/index.html?prefix=pod5_data/GM18501_R9/}.
To subset, call the \code{bam} file by \code{\link[Rsamtools]{scanBam}} function from
\code{\link[Rsamtools]{Rsamtools}} package and randomly select the 5 sequences.
and repackaging the subsetted \code{list} to the \code{bam} file for test running.
}
\description{
A \code{.bam} file containing the basecalling result of 5 sequences
containing with the MM, ML tag from \code{dorado}.
}
