% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeDMRsReplicatesNeighbourhood}
\alias{.computeDMRsReplicatesNeighbourhood}
\title{This function computes the differentially methylated regions between replicates
using the neighbourhood method.}
\usage{
.computeDMRsReplicatesNeighbourhood(
  methylationData,
  condition,
  regions = NULL,
  context = "CG",
  pseudocountM = 1,
  pseudocountN = 2,
  pValueThreshold = 0.01,
  minCytosinesCount = 4,
  minProportionDifference = 0.4,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  BPPARAM = BPPARAM,
  cores = cores
)
}
\description{
This function computes the differentially methylated regions between replicates
using the neighbourhood method.
}
