% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeVMDs.R
\name{.analyseReadsInsideRegionsVMDs}
\alias{.analyseReadsInsideRegionsVMDs}
\title{Analyse reads inside regions with computing mean and standard deviation per-read proportions}
\usage{
.analyseReadsInsideRegionsVMDs(methylationData, regions)
}
\arguments{
\item{methylationData}{A \code{GRanges} of methylation calls. Must include metadata:
\describe{
  \item{readsM, readsN}{methylated and total read counts in condition}
  \item{ONT_Cm, ONT_C}{per-read methylation and coverage indices (character vectors)}
}}

\item{regions}{A \code{GRanges} of genomic intervals to summarise.}
}
\value{
A \code{GRanges} with the same ranges as \code{regions}, plus these metadata:
\describe{
  \item{sumReadsM}{total methylated reads in condition}
  \item{sumReadsN}{total reads in condition}
  \item{proportion}{methylation proportion (sumReadsM/sumReadsN)}
  \item{cytosinesCount}{number of cytosines observed in each region}
  \item{mean}{mean value comparing per‐read proportions}
  \item{sd}{standard deviation comparing per‐read proportions}
  \item{w_mean}{weighted mean value comparing per‐read proportions}
  \item{w_sd}{weighted standard deviation comparing per‐read proportions}
}
}
\description{
Performs the analysis in all regions in a \code{\link{GRanges}} object,
computing per-region mean and standard deviation of proportions.
}
\author{
Radu Zabet and Young Jun Kim
}
