% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateMethylationData}
\alias{.validateMethylationData}
\title{Validate methylation data}
\usage{
.validateMethylationData(
  methylationData,
  variableName = "methylationData",
  manageDuplicates = "mean"
)
}
\arguments{
\item{methylationData}{the methylation data stored as a \code{\link{GRanges}}
object containing all the replicates.}
}
\description{
Checks whether the passed parameter has the correct format for methylation data
}
\author{
Alessandro Pio Greco and Nicolae Radu Zabet
}
