% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateMethylationProfile}
\alias{.validateMethylationProfile}
\title{Validate methylation profile}
\usage{
.validateMethylationProfile(methylationProfile)
}
\arguments{
\item{methylationProfile}{a \code{GRangesList} object
(see \code{\link{computeMethylationProfile}}).}
}
\description{
Checks whether the passed parameter is a \code{GRangesList} object that
represents the methylation profile
}
\author{
Radu Zabet
}
