% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.isInteger}
\alias{.isInteger}
\title{.is.integer}
\usage{
.isInteger(x, positive = FALSE)
}
\arguments{
\item{x}{the variable}

\item{positive}{a logical variable indicating whether to check if the number
is a positive integer}
}
\value{
a logical number whether the variable x is an integer or not
}
\description{
This function checks is a variable is an integer number
}
\author{
Radu Zabet
}
