% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.smartMergeDMRs}
\alias{.smartMergeDMRs}
\title{This takes a list of DMRs and attempts to merge DMRs while keeping the new 
DMRs statistically significant}
\usage{
.smartMergeDMRs(
  DMRs,
  minGap,
  respectSigns = TRUE,
  methylationData,
  minProportionDifference = 0.4,
  minReadsPerCytosine = 4,
  pValueThreshold = 0.01,
  test = "fisher",
  alternative = "two.sided",
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This takes a list of DMRs and attempts to merge DMRs while keeping the new 
DMRs statistically significant
}
