% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{DMRsNoiseFilterCG}
\alias{DMRsNoiseFilterCG}
\title{The DMRs between WT and met1-3 in CG context}
\format{
The \code{\link[GenomicRanges]{GRanges}} element contain 11 metadata columns;
see \code{\link{computeDMRs}}
}
\description{
A \code{GRangesList} object containing the DMRs between  Wild Type (WT) and
met1-3 mutant (met1-3) in Arabidopsis thaliana
(see \code{\link{methylationDataList}}). The DMRs were computed on the first
1 Mbp from Chromosome 3 with noise filter method using a triangular kernel
and a windowSize of 100 bp
}
\seealso{
\code{\link{filterDMRs}}, \code{\link{computeDMRs}},
\code{\link{analyseReadsInsideRegionsForCondition}}
and \code{\link{mergeDMRsIteratively}}
}
