% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.smartMergePMDs}
\alias{.smartMergePMDs}
\title{This takes a list of PMDs and attempts to merge PMDs while keeping the new 
PMDs statistically significant}
\usage{
.smartMergePMDs(
  PMDs,
  minGap,
  respectSigns = TRUE,
  methylationData,
  minReadsPerCytosine = 4,
  minMethylation = 0.4,
  maxMethylation = 0.6,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This takes a list of PMDs and attempts to merge PMDs while keeping the new 
PMDs statistically significant
}
