% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.countTotalNumberOfCytosines}
\alias{.countTotalNumberOfCytosines}
\title{Count total number of Cytosines}
\usage{
.countTotalNumberOfCytosines(methylationData, regions)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata 
columns: see \code{\link{methylationDataList}}}

\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object with the identified regions}
}
\value{
a the total number of Cytosines
}
\description{
Counts the number of cytosines in all the regions
}
\author{
Radu Zabet
}
