% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{.fisherTest}
\alias{.fisherTest}
\title{Fisher test}
\usage{
.fisherTest(m1, n1, m2, n2, alternative = c("two.sided", "less", "greater"))
}
\arguments{
\item{m1}{the number of methylated reads in condition 1}

\item{n1}{the total number of methylated reads in condition 1}

\item{m2}{the number of methylated reads in condition 2}

\item{n2}{the total number of methylated reads in condition 2}

\item{alternative}{indicates the alternative hypothesis and must be one of
\code{"two.sided"}, \code{"greater"} or \code{"less"}. You can specify just
the initial letter. Only used in the 2 by 2 case.}
}
\value{
The p-values of Fisher's exact test.
}
\description{
This function computes the p-values of the Fisher's exact test.
}
\author{
Radu Zabet
}
