% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeVMDs.R
\name{.computeVMDsBins}
\alias{.computeVMDsBins}
\title{This function computes the variance methylated regions 
using the bins method.}
\usage{
.computeVMDsBins(
  methylationData,
  regions = NULL,
  context = "CG",
  binSize = 100,
  minCytosinesCount = 4,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  sdCutoffMethod = "per.high",
  percentage = 0.05,
  cores = 1,
  parallel = parallel,
  BPPARAM = BPPARAM
)
}
\description{
This function computes the variance methylated regions 
using the bins method.
}
