% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{methylationDataList}
\alias{methylationDataList}
\title{The methylation data list}
\format{
The \code{GRanges} elements contain four metadata columns
\describe{
 \item{context}{the context in which the DMRs are computed (\code{"CG"},
               \code{"CHG"} or \code{"CHH"}).}
 \item{readsM}{the number of methylated reads.}
 \item{readsN}{the total number of reads.}
 \item{trinucleotide_context}{the specific context of the cytosine (H is
                             replaced by the actual nucleotide).}
}
}
\source{
Each element was created by by calling \code{\link{readBismark}}
function on the CX report files generated by Bismark
\url{http://www.bioinformatics.babraham.ac.uk/projects/bismark/}
for \url{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSM980986} dataset
in the case of Wild Type (WT) and
\url{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSM981032}
dataset in the case of met1-3 mutant (met1-3).
}
\description{
A \code{GRangesList} object containing the methylation data at each cytosine
location in the genome in  Wild Type (WT) and  met1-3 mutant (met1-3) in
Arabidopsis thaliana. The data only contains the first 1 Mbp from Chromosome 3.
}
