% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{.scoreTest}
\alias{.scoreTest}
\title{Score test}
\usage{
.scoreTest(m1, n1, m2, n2)
}
\arguments{
\item{m1}{the number of methylated reads in condition 1}

\item{n1}{the total number of methylated reads in condition 1}

\item{m2}{the number of methylated reads in condition 2}

\item{n2}{the total number of methylated reads in condition 2}
}
\value{
The p-values of the Score test
}
\description{
This function computes the p-values of the Score test.
}
\author{
Radu Zabet
}
