% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateMethylationDataList}
\alias{.validateMethylationDataList}
\title{Validate methylation data}
\usage{
.validateMethylationDataList(methylationDataList)
}
\arguments{
\item{methylationData}{the methylation data stored as a \code{\link{GRanges}}
object with six metadata columns (see \code{\link{methylationData}}).}
}
\description{
Checks whether the passed parameter has the correct format for methylation data
}
\author{
Radu Zabet
}
