% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateCoMethylationAlternative}
\alias{.validateCoMethylationAlternative}
\title{Validate alternative value for Co-methylation}
\usage{
.validateCoMethylationAlternative(alternative)
}
\arguments{
\item{test}{the alternative valuet used to Fisher exact's test must be one of
\code{"two.sided"}, \code{"greater"} or \code{"less"}. You can specify just
the initial letter.}
}
\description{
Checks whether the passed parameter is the alternative for detecting Co-methylation
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
