% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/movingAverage.R
\name{.movingAverage}
\alias{.movingAverage}
\title{Moving average}
\usage{
.movingAverage(
  minPos,
  maxPos,
  pos,
  val,
  weights = 1,
  windowSizeHalf = 150,
  normalize = FALSE,
  kernelFunction = "triangular",
  lambda = 0.5
)
}
\arguments{
\item{minPos}{the first position of the input fragment}

\item{maxPos}{the last position of the input fragment}

\item{pos}{the positions where the vector has values}

\item{val}{the values of the input vector}

\item{weights}{the weights associated with the moving average}

\item{normalize}{\code{logical} value that indicates whether the moving 
average is normalised}

\item{kernelFunction}{a \code{character} indicating which kernel function to 
be used. Can be one of \code{"uniform"}, \code{"triangular"}, 
\code{"gaussian"} or \code{"epanechnicov"}.}

\item{lambda}{numeric value required for the Gaussian filter 
(\code{K(x) = exp(-lambda*x^2)})}

\item{windowSize}{the size of the moving average}
}
\value{
A numeric vector with the values computed by the moving average
}
\description{
This function computes the moving average for a vector that has missing 
values.
}
\author{
Radu Zabet
}
