% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{PMDsNoiseFilterCG}
\alias{PMDsNoiseFilterCG}
\title{The PMDs between GM18501 and GM18876 using Noise_filter method}
\format{
The \code{\link[GenomicRanges]{GRanges}} element contain 5 metadata columns;
see \code{\link{computePMDs}}
}
\description{
A \code{GRangesList} object containing the PMDs between GM18501 and
GM18876 B-Lymphocyte cell lines in Homo sapiens
(see \code{\link{ontSampleGRangesList}}). The PMDs were computed on the
1.5 ~ 2 Mbp from Chromosome 1 with noise filter method using a triangular kernel
and a windowSize of 100 bp
}
\seealso{
\code{\link{filterPMDs}}, \code{\link{computePMDs}},
\code{\link{analyseReadsInsideRegionsForConditionPMD}}
and \code{\link{mergePMDsIteratively}}
}
