% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.analyseReadsInsideRegionsOneSample}
\alias{.analyseReadsInsideRegionsOneSample}
\title{Analyse reads inside regions}
\usage{
.analyseReadsInsideRegionsOneSample(methylationData, regions)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with four metadata 
columns; see \code{\link{MethylationDataList}}.}

\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object with the identified regions}
}
\value{
a \code{\link[GenomicRanges]{GRanges}} object with eaual sized tiles of the regions. 
The object consists of the following metadata.
\describe{
 \item{sumReadsM}{the number of methylated reads in condition 1}
 \item{sumReadsN}{the total number of reads in condition 1}
 \item{Proportion}{the proportion of methylated reads in condition 1}
}
}
\description{
Performs the analysis in all regions in a \code{\link[GenomicRanges]{GRanges}} object
}
\author{
Radu Zabet
}
