% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.printGenomicRanges}
\alias{.printGenomicRanges}
\title{Print GenomicRanges}
\usage{
.printGenomicRanges(gr)
}
\arguments{
\item{gr}{the GenomicRanges object}
}
\value{
a vector of type \code{character} with template chr:start..end
}
\description{
This function that prints a GenomicRanges object
}
\author{
Radu Zabet
}
