% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/movingAverage.R
\name{.movingSum}
\alias{.movingSum}
\title{Moving sum}
\usage{
.movingSum(
  minPos,
  maxPos,
  pos,
  val,
  weights = 1,
  windowSize = 150,
  normalize = FALSE
)
}
\arguments{
\item{minPos}{the first position of the input fragment}

\item{maxPos}{the last position of the input fragment}

\item{pos}{the positions where the vector has values}

\item{val}{the values of the input vector}

\item{weights}{the weights associated with the moving sum}

\item{windowSize}{the size of the moving sum}

\item{normalize}{\code{logical} value that indicates whether the moving sum is normalised}
}
\value{
A numeric vector with the values computed by the moving sum
}
\description{
This function computes the moving sum for a vector that has missing values.
}
\author{
Radu Zabet
}
