% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateBamfile}
\alias{.validateBamfile}
\title{Validate BAM Filename}
\usage{
.validateBamfile(bamfile)
}
\arguments{
\item{bamfile}{Character scalar. Path to a BAM file.}
}
\value{
Invisibly `TRUE` if file exists and extension is ".bam"; otherwise errors.
}
\description{
Checks that `bamfile` exists and has a ".bam" extension.
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
