% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateContext}
\alias{.validateContext}
\title{Validate context}
\usage{
.validateContext(context)

.validateContext(context)
}
\arguments{
\item{context}{the context in which the DMRs are computed (\code{"CG"},
\code{"CHG"} or \code{"CHH"})}

\item{length}{the expected length of the vector. If NULL any length is
allowed}

\item{test}{the  modified context used to call methylation information from bamfile must be one of
sequence context for \code{selectCytosine()} (e.g. \code{"CG"}, \code{"CHG"}, \code{"CHH"})}
}
\description{
Checks whether the passed parameter is the context

Checks whether the passed parameter is the modified context
}
\author{
Radu Zabet

Radu Zabet and Young Jun Kim
}
\keyword{internal}
