% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{ont_gr_GM18870_chr1_sorted_bins_1k}
\alias{ont_gr_GM18870_chr1_sorted_bins_1k}
\title{The ONT methylation data example}
\format{
The \code{GRanges} elements contain four
  additional metadata columns:
  \describe{
    \item{ONT_Cm}{comma-delimited read‐indices called modified}
    \item{ONT_C}{comma-delimited read‐indices covering but unmodified}
    \item{readsM}{integer count of modified reads per site}
    \item{readsN}{integer count of same‐strand reads covering each site}
  }
}
\source{
data from https://genome.cshlp.org/content/34/11/2061.
}
\description{
A \code{GRanges} object containing cytosine sites, annotated with
  per-site ONT methylation calls
}
