% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadData.R
\name{.joinMethylationData}
\alias{.joinMethylationData}
\title{Join methylation data}
\usage{
.joinMethylationData(
  cx1,
  cx2,
  sample1_name = "Sample1",
  sample2_name = "Sample2"
)
}
\arguments{
\item{cx1}{a \code{\link{GRanges}} object of CX reports (Bismark) with three
metadata columns the cytosine context (CG, CHG or CHH), the number of
methylated reads and the total number of reads at that position in condition
1}

\item{cx2}{a \code{\link{GRanges}} object of CX reports (Bismark) with three
metadata columns the cytosine context (CG, CHG or CHH), the number of
methylated reads and the total number of reads at that position in condition
2}
}
\value{
a the methylation data stored as a \code{\link{GRanges}}
object with six metadata columns (see \code{\link{methylationDataList}}).
}
\description{
This function takes as input two \code{\link{GRanges}} objects of CX reports
(Bismark) for two conditions and  returns a \code{\link{GRanges}} object with
five metadata columns
}
