% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeBetaRegSingle}
\alias{.computeBetaRegSingle}
\title{Beta regression}
\usage{
.computeBetaRegSingle(y, condition)
}
\arguments{
\item{y}{vector containing the proportions (readsM/readsN) for a single cytosine.}

\item{condition}{vector containing the conditions of the experiment.}
}
\value{
The p-values of the beta regression test.
}
\description{
This function computes the p-values of the beta regression test
}
\author{
Alessandro Pio Greco and Nicolae Radu Zabet
}
