% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.analyseReadsInsideRegionsForCondition}
\alias{.analyseReadsInsideRegionsForCondition}
\title{Analyse reads inside regions}
\usage{
.analyseReadsInsideRegionsForCondition(
  regions,
  methylationData,
  label = "",
  context = ""
)
}
\arguments{
\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object with the identified regions}

\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata 
columns see \code{\link{methylationDataList}}}
}
\value{
a \code{\link[GenomicRanges]{GRanges}} object with eaual sized tiles of the regions. 
The object consists of the following metadata 
\describe{
 \item{sumReadsM}{the number of methylated reads in condition 1}
 \item{sumReadsN}{the total number of reads in condition 1}
 \item{proportion}{the proportion of methylated reads in condition 1}
 \item{cytosinesCount}{the number of cytosines in the correct context} 
}
}
\description{
Performs the analysis in all regions in a \code{\link[GenomicRanges]{GRanges}} object
}
\author{
Radu Zabet
}
