% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.analyseReadsInsideRegionsPMDs}
\alias{.analyseReadsInsideRegionsPMDs}
\title{Analyse reads inside regions}
\usage{
.analyseReadsInsideRegionsPMDs(methylationData, regions)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata 
columns see \code{\link{methylationDataList}}}

\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object with the identified regions}
}
\value{
a \code{\link[GenomicRanges]{GRanges}} object with eaual sized tiles of the regions. 
The object consists of the following metadata 
\describe{
 \item{sumReadsM}{the number of methylated reads}
 \item{sumReadsN}{the total number of reads}
 \item{proportion}{the proportion of methylated reads}
 \item{ONT_Cm}{comma-delimited modified read‐indices}
 \item{ONT_C}{comma-delimited read‐indices covering but unmodified}
 \item{cytosinesCount}{the number of cytosines in the correct context} 
}
}
\description{
Performs the analysis in all regions in a \code{\link[GenomicRanges]{GRanges}} object for 
PMD calculation
}
\author{
Radu Zabet and Young Jun Kim
}
