% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeDMRs.R
\name{.computeDMRsBins}
\alias{.computeDMRsBins}
\title{This function computes the differentially methylated regions between two conditions
using the bins method.}
\usage{
.computeDMRsBins(
  methylationData1,
  methylationData2,
  regions = NULL,
  context = "CG",
  binSize = 100,
  test = "fisher",
  pValueThreshold = 0.01,
  minCytosinesCount = 4,
  minProportionDifference = 0.4,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This function computes the differentially methylated regions between two conditions
using the bins method.
}
