% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{.computeaAjustedPValuesInDMRs}
\alias{.computeaAjustedPValuesInDMRs}
\title{Compute adjusted p-values in DMRs}
\usage{
.computeaAjustedPValuesInDMRs(test, DMRs, alternative = "two.sided")
}
\arguments{
\item{test}{a \code{character} indicating the statistical test used .
(\code{"fisher"} for Fisher's exact test or \code{"score"} for Score test).}

\item{DMRs}{a \code{\link{GRanges}} object with the DMRs.}

\item{alternative}{indicates the alternative hypothesis and must be one of
\code{"two.sided"}, \code{"greater"} or \code{"less"}. You can specify just
the initial letter. Only used in the 2 by 2 case. This is used only for
Fisher's test.}
}
\value{
The adjuested p-values of the statistical test.
}
\description{
This function computes the adjusted p-values (using Benjamini & Hochberg
method) using one the statistical tests
}
\author{
Radu Zabet
}
