% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.analyseReadsInsideBinsReplicates}
\alias{.analyseReadsInsideBinsReplicates}
\title{Analyse reads inside regions}
\usage{
.analyseReadsInsideBinsReplicates(
  methylationData,
  bins,
  currentRegion,
  condition,
  pseudocountM,
  pseudocountN
)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata
columns; see \code{\link{methylationDataList}}}

\item{currentRegion}{a \code{\link[GenomicRanges]{GRanges}} object with the identified regions}

\item{condition}{The vector containing the two conditions for the experiment.}

\item{pseudocountM}{numerical value to be added to the methylated reads
before modelling beta regression.}

\item{pseudocountN}{numerical value to be added to the total reads
before modelling beta regression.}
}
\value{
a \code{\link[GenomicRanges]{GRanges}} object with eaual sized tiles of the regions.
The object consists of the following metadata
\describe{
 \item{sumReadsM1}{the number of methylated reads in condition 1}
 \item{sumReadsN1}{the total number of reads in condition 1}
 \item{proportion1}{the proportion of methylated reads in condition 1}
 \item{sumReadsM2}{the number of methylated reads in condition 2}
 \item{sumReadsN2}{the total number of reads in condition 2}
 \item{proportion2}{the proportion of methylated reads in condition 2}
 \item{cytosinesCount}{the number of cytosines in the correct context}
}
}
\description{
Performs the analysis in equal width regions of an \code{\link[GenomicRanges]{GRanges}}
object
}
\author{
Nicolae Radu Zabet and Alessandro Pio Greco
}
