% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computePMDs.R
\name{.computePMDsBins}
\alias{.computePMDsBins}
\title{This function computes the partially methylated regions 
using the bins method.}
\usage{
.computePMDsBins(
  methylationData,
  regions = NULL,
  context = "CG",
  binSize = 100,
  minCytosinesCount = 4,
  minMethylation = 0.4,
  maxMethylation = 0.6,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This function computes the partially methylated regions 
using the bins method.
}
