% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.splitGRangesEqualy}
\alias{.splitGRangesEqualy}
\title{.splitGRangesEqualy}
\usage{
.splitGRangesEqualy(regions, breaks = 1)
}
\arguments{
\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object}

\item{breaks}{number of elements to break the GRanges}
}
\value{
a \code{list} consisting of GRanges objects of same total width
}
\description{
This function splits a set of GRanges into a list of equally width GRanges 
objects
}
\author{
Radu Zabet
}
