% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeAdjuestedPValuesReplicates}
\alias{.computeAdjuestedPValuesReplicates}
\title{Compute adjusted p-values}
\usage{
.computeAdjuestedPValuesReplicates(proportions, condition, cores)
}
\arguments{
\item{condition}{vector containing the conditions of the experiment.}

\item{cores}{the number of cores to be used to fit the model.}

\item{conditions}{matrix containing the proportions (readsM/readsN) for all the cytosines.}
}
\value{
The adjusted p-values of the statistical test.
}
\description{
This function computes the adjusted p-values (using Benjamini & Hochberg
method)
}
\author{
Nicolae Radu Zabet and Alessandro Pio Greco
}
