% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialCorrelation.R
\name{.computeMethylationDataSpatialCorrelation}
\alias{.computeMethylationDataSpatialCorrelation}
\title{Compute methylation data spatial correlation}
\usage{
.computeMethylationDataSpatialCorrelation(methylationData, regions, distance)
}
\arguments{
\item{methylationData}{the methylation data stored as a \code{\link[GenomicRanges]{GRanges}}
object with four metadata columns (see \code{\link{methylationDataList}}).}

\item{regions}{a \code{\link[GenomicRanges]{GRanges}} object with the regions where to
compute the correlation If NULL, the correlation is computed genome-wide.}

\item{distance}{the distance used when computing the correlation of
methylation levels}
}
\value{
correlation of the methylation levels for Cytosines located at a
certain distance apart.
}
\description{
This function computes the correlation of the methylation levels for
Cytosines located at a certain distance apart. The function returns a the
correlation of methylation levels at distance equal to the specified
thresholds.
}
\author{
Radu Zabet
}
