% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateChromosome}
\alias{.validateChromosome}
\title{Validate Chromosome Names}
\usage{
.validateChromosome(chr, genome)
}
\arguments{
\item{chr}{Character vector of chromosome names.}

\item{genome}{A validated BSgenome object.}
}
\value{
Invisibly `TRUE` if all `chr` are in `seqnames(genome)`; otherwise errors.
}
\description{
Checks that `chr` entries are present in the supplied BSgenome.
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
