% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computePMDs.R
\name{.computePMDsNeighbourhood}
\alias{.computePMDsNeighbourhood}
\title{This function computes the partially methylated domains
using the neighbourhood method. This assumes the computation of partially methylated 
cytosines followed by smart merging of these cytosines while keeping the new PMDs 
statistically significant.}
\usage{
.computePMDsNeighbourhood(
  methylationData,
  regions = NULL,
  context = "CG",
  minCytosinesCount = 4,
  minMethylation = 0.4,
  maxMethylation = 0.6,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This function computes the partially methylated domains
using the neighbourhood method. This assumes the computation of partially methylated 
cytosines followed by smart merging of these cytosines while keeping the new PMDs 
statistically significant.
}
