% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.stopIfNotAll}
\alias{.stopIfNotAll}
\title{.stopIfNotAll}
\usage{
.stopIfNotAll(exprs, errorMsg)
}
\arguments{
\item{errorMsg}{the error message to be printed}

\item{expr}{an array of logical expressions}
}
\description{
This function asses an expression and if this is FALSE stops the execution
and prints a message
}
\author{
Radu Zabet
}
