% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateCoMethylationStatTest}
\alias{.validateCoMethylationStatTest}
\title{Validate statistial test for Co-methylation}
\usage{
.validateCoMethylationStatTest(test)
}
\arguments{
\item{test}{the statistical test used to call Co-methylation (\code{"fisher"} for
Fisher's exact test, \code{"score"} for Score test or \code{"permutation"} for Permutation test).}
}
\description{
Checks whether the passed parameter is the statistical test for detecting Co-methylation
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
