% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateStatisticalTest}
\alias{.validateStatisticalTest}
\title{Validate statistial test}
\usage{
.validateStatisticalTest(test)
}
\arguments{
\item{test}{the statistical test used to call DMRs (\code{"fisher"} for
Fisher's exact test or \code{"score"} for Score test).}
}
\description{
Checks whether the passed parameter is the statistical test
}
\author{
Radu Zabet
}
