% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.convertResult}
\alias{.convertResult}
\title{This function extracts the p-value for each cytosine from the betareg object}
\usage{
.convertResult(x)
}
\description{
This function extracts the p-value for each cytosine from the betareg object
}
