% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{.fisherTestPValue}
\alias{.fisherTestPValue}
\title{Fisher test}
\usage{
.fisherTestPValue(x, alternative = c("two.sided", "less", "greater"))
}
\arguments{
\item{x}{a vector with 4 values}
}
\value{
The p-values of Fisher's exact test.
}
\description{
This function computes the p-values of the Fisher's exact test.
}
\author{
Radu Zabet
}
