% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMRcaller.R
\docType{data}
\name{ont_gr_GM18870_chr1_PMD_bins_1k}
\alias{ont_gr_GM18870_chr1_PMD_bins_1k}
\title{Partially Methilated Domains example}
\format{
The \code{GRanges} elements contain seven metadata columns:
\describe{
 \item{context}{the context in which the PMDs was computed (\code{"CG"},
 \code{"CHG"} or \code{"CHH"}).}
 \item{sumReadsM}{the number of methylated reads.}
 \item{sumReadsN}{the total number of reads.}
 \item{proportion}{the proportion methylated reads filtered between
 \code{minMethylation} and \code{maxMethylation}.}
 \item{cytosinesCount}{the number of cytosines in the PMDs.}
}
}
\source{
data from https://genome.cshlp.org/content/34/11/2061.
}
\description{
Partially methylated domains called on chr1 in GM18870 cells called in 1Kb
bins with \code{computePMDs} function.
}
