% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.mergePMDsIteratively}
\alias{.mergePMDsIteratively}
\title{This takes a list of PMDs and attempts to merge PMDs while keeping the new 
PMDs filtering by the preset parameter}
\usage{
.mergePMDsIteratively(
  PMDs,
  minGap,
  respectSigns = TRUE,
  methylationData,
  minReadsPerCytosine = 4,
  minMethylation = 0.4,
  maxMethylation = 0.6
)
}
\description{
This takes a list of PMDs and attempts to merge PMDs while keeping the new 
PMDs filtering by the preset parameter
}
