% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage.R
\name{.countMethylationDataCoverage}
\alias{.countMethylationDataCoverage}
\title{Count methylation data coverage}
\usage{
.countMethylationDataCoverage(methylationData, regions, threshold)
}
\arguments{
\item{methylationData}{the methylation data stored as a \code{\link{GRanges}}
object with four metadata columns (see \code{\link{methylationDataList}}).}

\item{regions}{a \code{\link{GRanges}} object with the regions where to
compute the coverage. If NULL, the coverage is computed genome-wide.}

\item{threshold}{the threshold used when counting cytosines that have at
least a certain number of reads}
}
\value{
a the number of cytosines with more reads than the threshold in the
specified region
}
\description{
This function computes the number of cytosines with more reads than a
threshold in the methylation data
}
\author{
Radu Zabet
}
