% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCoMethylatedRegions.R
\name{.validateCorrelationStatTest}
\alias{.validateCorrelationStatTest}
\title{Validate statistial test for Correlation between two Regions}
\usage{
.validateCorrelationStatTest(test)
}
\arguments{
\item{test}{the statistical test used to call Correlation Between Methylated Regions (CMRs) (\code{"pearson"},
\code{"kendall"} or \code{"spearman"} for Correlation test).}
}
\description{
Checks whether the passed parameter is the statistical test for detecting Correlation in Regions
}
\author{
Radu Zabet and Young Jun Kim
}
\keyword{internal}
