% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.computeBetaReg}
\alias{.computeBetaReg}
\title{This function applies the .computeBetaRegSingle to the entire dataset}
\usage{
.computeBetaReg(y, condition, cores)
}
\description{
This function applies the .computeBetaRegSingle to the entire dataset
}
