% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{.validateGRanges}
\alias{.validateGRanges}
\title{Validate GRanges}
\usage{
.validateGRanges(
  regions,
  methylationData,
  length = NULL,
  generateGenomeWide = TRUE,
  variableName = "regions",
  minLength = 0
)
}
\arguments{
\item{regions}{a \code{\link{GRanges}} object. If \code{NULL} and
\code{generateGenomeWide} is true it uses the \code{methylationData} to
compute the regions and returns this \code{\link{GRanges}} object}

\item{methylationData}{the methylation data stored as a \code{\link{GRanges}}
object with six metadata columns (see \code{\link{methylationData}}).}

\item{length}{the expected length of the vector. If \code{NULL} any length is
allowed.}

\item{generateGenomeWide}{logical value to indicate whether to compute the
regions that span over all the \code{methylationData}}
}
\value{
a \code{\link{GRanges}} object representing the regions
}
\description{
Checks whether the passed parameter is a \code{\link{GRanges}} object
}
\author{
Radu Zabet
}
