% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localProfile.R
\name{.plotGeneticElements}
\alias{.plotGeneticElements}
\title{Plot genetic elements}
\usage{
.plotGeneticElements(gff, region, col)
}
\arguments{
\item{gff}{a \code{\link{GRanges}} object with all elements usually imported
from a GFF3 file}

\item{region}{the location on the genome where to draw the genetic elements}

\item{col}{the colors used to draw the exons and the transposable elemnets}
}
\description{
This function draws the genetic elements over a plot
}
\author{
Radu Zabet
}
