% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regions.R
\name{.analyseReadsInsideBinsOneSample}
\alias{.analyseReadsInsideBinsOneSample}
\title{Analyse reads inside regions for one sample}
\usage{
.analyseReadsInsideBinsOneSample(methylationData, bins, currentRegion)
}
\arguments{
\item{methylationData}{a \code{\link[GenomicRanges]{GRanges}} object with five metadata 
columns; see \code{\link{methylationDataList}}.}

\item{bins}{a \code{\link[GenomicRanges]{GRanges}} object with the bins}
}
\value{
a \code{\link[GenomicRanges]{GRanges}} object with eaual sized tiles of the regions. 
The object consists of the following metadata 
\describe{
 \item{sumReadsM}{the number of methylated reads}
 \item{sumReadsN}{the total number of reads}
 \item{Proportion}{the proportion of methylated reads}
}
}
\description{
Performs the analysis in equal width regions of an \code{\link[GenomicRanges]{GRanges}} 
object
}
\author{
Radu Zabet
}
