% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Replicates.R
\name{.validateStatisticalTestReplicates}
\alias{.validateStatisticalTestReplicates}
\title{Validate statistial test}
\usage{
.validateStatisticalTestReplicates(test)
}
\arguments{
\item{test}{the statistical test used to call DMRs (\code{"betareg"} for
Beta regression.}
}
\description{
Checks whether the passed parameter is the statistical test
}
\author{
Alessandro Pio Greco and Nicolae Radu Zabet
}
