% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computePMDs.R
\name{.computePMDsNoiseFilter}
\alias{.computePMDsNoiseFilter}
\title{This function computes the partially methylated regions 
using the noise filter method.}
\usage{
.computePMDsNoiseFilter(
  methylationData,
  regions = NULL,
  context = "CG",
  windowSize = 100,
  kernelFunction = "triangular",
  lambda = 0.5,
  minCytosinesCount = 4,
  minMethylation = 0.4,
  maxMethylation = 0.6,
  minGap = 200,
  minSize = 50,
  minReadsPerCytosine = 4,
  cores = 1,
  BPPARAM = BPPARAM
)
}
\description{
This function computes the partially methylated regions 
using the noise filter method.
}
